@@ -46,7 +46,7 @@
 
 /*
  * These are needed because there is no "official" way to specify
- * WHERE to save the thread state.
+ * WHERE to save the thread state. (At least not until Python 2.3)
  */
 #ifdef WITH_THREAD
 #  define MY_BEGIN_ALLOW_THREADS(st)    \
@@ -260,7 +260,7 @@ PyObject* pysqlite_connect(PyObject *self, PyObject *args, PyObject *kwargs)
         if (errmsg != NULL)
         {
             PyErr_SetString(_sqlite_DatabaseError, errmsg);
-            free(errmsg);
+            sqlite_freemem(errmsg);
         }
         else
         {
@@ -517,8 +517,8 @@ static void aggregate_finalize(sqlite_func *context)
     }
     else
     {
-        Py_DECREF(function_result);
         s = PyObject_Str(function_result);
+        Py_DECREF(function_result);
         sqlite_set_result_string(context, PyString_AsString(s), -1);
         Py_DECREF(s);
     }
@@ -548,6 +548,7 @@ static int sqlite_busy_handler_callback(void* void_data, const char* tablename,
     MY_END_ALLOW_THREADS(con->tstate)
 
     args = PyTuple_New(3);
+    Py_INCREF(userdata);
     PyTuple_SetItem(args, 0, userdata);
     PyTuple_SetItem(args, 1, PyString_FromString(tablename));
     PyTuple_SetItem(args, 2, PyInt_FromLong((long)num_busy));
@@ -755,8 +756,6 @@ static PyObject* _con_set_command_logfile(pysqlc* self, PyObject *args, PyObject
 
     if (logfile == Py_None)
     {
-        Py_DECREF(logfile);
-
         Py_INCREF(Py_None);
         return Py_None;
     }
@@ -1067,7 +1066,7 @@ static int _seterror(int returncode,  char *errmsg)
         default:
             PyErr_SetString(_sqlite_DatabaseError, (errmsg!=NULL)?errmsg:sqlite_error_string(returncode));
     }
-    free(errmsg);
+    sqlite_freemem(errmsg);
     return returncode;
 }
 
@@ -1239,7 +1238,7 @@ int process_record(void* p_data, int num_fields, char** p_fields, char** p_col_n
     PyObject* type_code;
 
     PyObject* expected_types;
-    PyObject* expected_type_name;
+    PyObject* expected_type_name = NULL;
     PyObject* converters;
     PyObject* converted;
     PyObject* callable;
