@@ -246,6 +246,12 @@ class moduleTestCases(unittest.TestCase, testsupport.TestSupport):
             self.cur.execute("select max(%s, %s)", (4, 5))
         except StandardError, msg:
             self.fail(msg)
+
+        # Empty tuple
+        try:
+            self.cur.execute("select 3+4", ())
+        except StandardError, msg:
+            self.fail(msg)
         self.cur.close()
 
     def CheckExecuteWithDictionary(self):
